/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.bundu.bezugswst;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.dive4elements.river.artifacts.bundu.BunduResultType;
import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.AbstractExportContext;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;
import org.dive4elements.river.model.BedHeightValueType;

/**
 * Contains the results of a //TODO LINK{@link BezugswstCalculation}.
 *
 * @author Gernot Belger
 */
final class BezugswstMissVolCalculationResult2 extends AbstractCalculationExportableResult implements BunduWstExporter.MissResultType {

    private static final long serialVersionUID = 1L;

    private static final String JASPER_FILE = "/jasper/templates/bundu.bezugswst.result2.jrxml";
    private static final String fieldVolumeMsgKey = "bundu.export.bezugswst.csv.meta.miss.volume.field";
    private static final String fieldMassMsgKey = "bundu.export.bezugswst.csv.meta.miss.mass.field";

    public BezugswstMissVolCalculationResult2(final String label, final Collection<ResultRow> rows) {
        super(label, rows);

    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        // no metadata
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    protected String[] formatRow(final AbstractExportContext exportContext, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(20);

        lines.add(exportContext.formatRowValue(row, GeneralResultType.station));
        @SuppressWarnings("unchecked")
        final List<Double> vFields = (List<Double>) row.getValue(BunduResultType.missVolumeFields);
        assert (vFields.size() == fieldSize()); // immer abgleichen mit der Calculation!
        @SuppressWarnings("unchecked")
        final List<Double> mFields = (List<Double>) row.getValue(BunduResultType.missMassFields);
        assert (vFields.size() == mFields.size());
        for (int i = 0; i < vFields.size(); i++) {
            lines.add(BunduResultType.missVolumeFields.exportValue(exportContext.getContext(), vFields.get(i)));
            lines.add(BunduResultType.missMassFields.exportValue(exportContext.getContext(), mFields.get(i)));
        }
        lines.add(exportContext.formatRowValue(row, BunduResultType.missVolumeMeanBed));
        lines.add(exportContext.formatRowValue(row, BunduResultType.missMassMeanBed));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final int colSize = 2 * fieldSize() + 3;
        exportContextCSV.writeTitleForTabs("bundu.export.csv.title.bezugswst.result2", colSize); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));

        for (int i = 1, j = 1; i <= fieldSize() * 2; i += 2, j++) {
            header.add(exportContextCSV.msg(fieldVolumeMsgKey, j));
            header.add(exportContextCSV.msg(fieldMassMsgKey, j));
        }
        header.add(exportContextCSV.formatCsvHeader(BunduResultType.missVolumeMeanBed));
        header.add(exportContextCSV.formatCsvHeader(BunduResultType.missMassMeanBed));

        exportContextCSV.writeCSVLine(header.toArray(new String[header.size()]));

    }

    /**
     * Number of elements of the various field lists
     */
    private int fieldSize() {
        return BedHeightValueType.FIELD_LAST_INDEX - BedHeightValueType.FIELD_FIRST_INDEX + 1;
    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {

        return this.formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextPDF, final ResultRow row) {

        return this.formatRow(exportContextPDF, row);
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContextPDF, final MetaAndTableJRDataSource source) {
        /* column headings */

        // TODO Auftrennung jeder Volumen+Masse-Spalte in zwei getrennte Volumen- und Masse-Spalten
        exportContextPDF.addJRMetadata(source, "station_header", GeneralResultType.station);
        for (int i = 1; i <= fieldSize(); i++) {
            exportContextPDF.addJRMetadata(source, "field_header" + String.valueOf(i), exportContextPDF.msg(fieldVolumeMsgKey, i));
            exportContextPDF.addJRMetadata(source, "field_header" + String.valueOf(i), exportContextPDF.msg(fieldMassMsgKey, i));
        }
        exportContextPDF.addJRMetadata(source, "mean_bedheight_header", BunduResultType.missVolumeMeanBed);
        exportContextPDF.addJRMetadata(source, "mean_bedheight_header", BunduResultType.missMassMeanBed);

    }
}